package com.elsevier.example.music.quarkus.reader;

import com.elsevier.example.music.quarkus.ChartEntry;
import com.elsevier.example.music.quarkus.codec.ChartEntryDecoder;

import java.io.IOException;
import java.io.InputStream;
import java.io.UncheckedIOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.stream.Stream;

public class LocalFileChartEntryReader implements ChartEntryReader{

    private final Path file;
    private final ChartEntryDecoder chartEntryDecoder;
    private InputStream inputStream;

    public LocalFileChartEntryReader(Path file, ChartEntryDecoder chartEntryDecoder) {
        this.file = file;
        this.chartEntryDecoder = chartEntryDecoder;
    }

    @Override
    public Stream<ChartEntry> getStream() {
        try {
            inputStream = Files.newInputStream(file);
        } catch (IOException e) {
            throw new UncheckedIOException(e);
        }
        return chartEntryDecoder.decode(inputStream);
    }

    @Override
    public void close() {
        if (inputStream != null) {
            try {
                inputStream.close();
            } catch (IOException e) {
                throw new UncheckedIOException(e);
            }
        }
    }
}
