package com.elsevier.example.music.quarkus.reader;

import com.elsevier.example.music.quarkus.ChartEntry;
import com.elsevier.example.music.quarkus.codec.ChartEntryDecoder;
import software.amazon.awssdk.services.s3.S3Client;
import software.amazon.awssdk.services.s3.model.GetObjectRequest;

import java.io.IOException;
import java.io.InputStream;
import java.io.UncheckedIOException;
import java.util.stream.Stream;

public class S3ChartEntryReader implements ChartEntryReader {
    private final S3Client s3Client;
    private final String bucket;
    private final String key;
    private final ChartEntryDecoder chartEntryDecoder;
    private InputStream inputStream;

    public S3ChartEntryReader(S3Client s3Client, String bucket, String key, ChartEntryDecoder chartEntryDecoder) {
        this.s3Client = s3Client;
        this.bucket = bucket;
        this.key = key;
        this.chartEntryDecoder = chartEntryDecoder;
    }

    @Override
    public Stream<ChartEntry> getStream() {
        GetObjectRequest getObjectRequest = GetObjectRequest
                .builder()
                .key(key)
                .bucket(bucket)
                .build();
        inputStream = s3Client.getObject(getObjectRequest);
        return chartEntryDecoder.decode(inputStream);
    }

    @Override
    public void close() {
        if (inputStream != null) {
            try {
                inputStream.close();
            } catch (IOException e) {
                throw new UncheckedIOException(e);
            }
        }
    }
}
